import os
from pathlib import Path
from subprocess import CalledProcessError, check_output


def git(command):
    """
    run a git command
    """
    command = ["git"] + command
    try:
        return check_output(command, text=True).strip()
    except CalledProcessError:
        raise OSError(
            "Something went wrong while running:\n" f"{' '.join(command)}"
        )


def get_git_top_level_path():
    return Path(git(["rev-parse", "--show-toplevel"]))


def get_git_dir():
    return Path(git(["rev-parse", "--git-dir"]))


def get_attrfile(scope):
    if scope == "global":
        config_dir = Path("~/.config").expanduser()
        xdg_config_dir = Path(os.environ.get("XDG_CONFIG_DIR", config_dir))
        if xdg_config_dir.exists():
            attrfile = xdg_config_dir / "git" / "attributes"
        else:
            attrfile = Path("~/.gitconfig")
    else:
        attrfile = get_git_dir() / "info" / "attributes"

    return attrfile.expanduser()



import sys

from .git import get_attrfile, git
from .stripout import patterns

attr_lines = [
    f"{pattern} {attribute}\n"
    for attribute in ["filter=dotenvstripout"]
    for pattern in patterns
]


def _install(scope="local"):
    python = sys.executable.replace("\\", "/")
    git(["config", f"--{scope}", "filter.dotenvstripout.smudge", "cat"])
    git(
        [
            "config",
            f"--{scope}",
            "filter.dotenvstripout.clean",
            f'"{python}" -m dotenv_stripout --stdin',
        ]
    )

    attrfile = get_attrfile(scope)

    if not attrfile.exists():
        attrfile.touch(exist_ok=True)

    with attrfile.open("r") as f:
        attrs = f.readlines()

    with attrfile.open(
        "a",
    ) as f:
        if f.tell():
            f.write("\n")
        for line in attr_lines:
            if line not in attrs:
                f.write(line)


def _uninstall(scope="local"):
    git(["config", f"--{scope}", "--remove-section", "filter.dotenvstripout"])
    attrfile = get_attrfile(scope)
    with attrfile.open("r") as f:
        attrs_to_keep = [
            line for line in f.readlines() if line not in attr_lines
        ]
    with attrfile.open("w") as f:
        f.writelines(attrs_to_keep)


def is_installed(scope="local"):
    attrfile = get_attrfile(scope)
    try:
        with attrfile.open("r") as f:
            attrs = f.readlines()
        if all([line in attrs for line in attr_lines]):
            return True
        else:
            return False
    except FileNotFoundError:
        return False



import typer

from . import __version__
from .install import _install, _uninstall, is_installed
from .stripout import list_dotenv_file_paths, strip_file, strip_stdin

cli = typer.Typer(help="Strip secrets from all .env files in the current repo")


@cli.callback(invoke_without_command=True)
def main(
    ctx: typer.Context,
    dry_run: bool = typer.Option(
        False,
        help="Show the effect of the command without running it",
    ),
    stdin: bool = typer.Option(
        False,
        help="Read lines from stdin and write stripped lines to stdout",
    ),
    version: bool = typer.Option(
        False,
        "-v",
        help="Print the package version",
    ),
):
    if ctx.invoked_subcommand is None:
        if version:
            typer.echo(__version__)
            raise typer.Exit(0)
        try:
            if stdin:
                strip_stdin()
            else:
                paths = list_dotenv_file_paths()
                if dry_run:
                    typer.echo("Dry run - Would have stripped secrets from:")
                    for path in paths:
                        typer.echo(path)
                elif typer.confirm(
                    "Are you sure you want to strip secrets from this repo?"
                ):
                    typer.echo("Stripping secrets from:")
                    for path in paths:
                        typer.echo(path)
                        strip_file(path)
                else:
                    raise typer.Abort()

        except OSError as e:
            typer.echo(e)
            raise typer.Exit(1)


@cli.command(help="Check whether the filter has been installed")
def status(
    _global: bool = typer.Option(
        False,
        "--global",
        help=(
            "If set, the command will check for the filter in the "
            "global git config instead of the current repo"
        ),
    ),
):
    scope = "global" if _global else "local"
    if is_installed(scope):
        typer.echo(f"Filter is installed {scope}ly")
    else:
        typer.echo(f"Filter is not installed {scope}ly")
        if typer.confirm("Would you like to install it?"):
            _install(scope)
            typer.echo("Done!")
        else:
            raise typer.Abort()


@cli.command(help="Install dotenv-stripout as a git filter")
def install(
    _global: bool = typer.Option(
        False,
        "--global",
        help=(
            "If set, the filter will be added to your global git "
            "config instead of the current repo"
        ),
    ),
):
    scope = "global" if _global else "local"
    if is_installed(scope):
        typer.echo(f"Filter is already {scope}ly installed!")
        raise typer.Exit(1)
    else:
        _install(scope)
        typer.echo("Done!")


@cli.command(help="Uninstall the filter")
def uninstall(
    _global: bool = typer.Option(
        False,
        "--global",
        help=(
            "If set, the filter will be removed from your global git "
            "config instead of the current repo"
        ),
    ),
):
    scope = "global" if _global else "local"
    if is_installed(scope):
        _uninstall(scope)
        typer.echo("Done!")
    else:
        typer.echo(f"Filter is not yet {scope}ly installed!")
        raise typer.Exit(1)


if __name__ == "__main__":
    cli()



import sys

from .git import get_git_top_level_path

patterns = ["*.env", "*.env.*"]


def list_dotenv_file_paths():
    repo_path = get_git_top_level_path()
    return [
        path for pattern in patterns for path in list(repo_path.rglob(pattern))
    ]


def strip_line(line, newline=""):
    line = line.strip()
    if len(line) > 0:
        line = line.split("=")[0] + f"={newline}"
    return line


def strip_lines(lines, newline=""):
    return [strip_line(line, newline=newline) for line in lines]


def strip_file(path):
    with path.open("r") as f:
        lines = f.readlines()
    stripped_lines = strip_lines(lines, newline="\n")
    with path.open("w") as f:
        f.writelines(stripped_lines)


def strip_stdin():
    for line in sys.stdin:
        sys.stdout.write(strip_line(line, newline="\n"))



from pathlib import Path

from dotenv_stripout.stripout import list_dotenv_file_paths

data_dir = Path(__file__).parent / "data"
names_to_match = [".env", ".env.something", "something.env.something"]
names_to_not_match = [".environment"]
directories = [data_dir, data_dir / "subdir", data_dir / "subdir" / "subsubdir"]
paths_to_match = [dir / name for dir in directories for name in names_to_match]
paths_to_not_match = [
    dir / name for dir in directories for name in names_to_not_match
]

found_paths = set(list_dotenv_file_paths())


def test_all_correct_paths_found():
    for path in paths_to_match:
        assert path in found_paths


def test_no_incorrect_paths_found():
    for path in paths_to_not_match:
        assert path not in found_paths



from dotenv_stripout.stripout import strip_line


def test_normal_line():
    input_line = "MY_SECRET_PASSWORD=IqLTLrFviwHTDKWGZoR7uB2JtM1wjwE34MBwoztE"
    expected_output_line = "MY_SECRET_PASSWORD="
    output_line = strip_line(input_line)
    assert output_line == expected_output_line


def test_line_without_value():
    input_line = "MY_SECRET_PASSWORD="
    expected_output_line = "MY_SECRET_PASSWORD="
    output_line = strip_line(input_line)
    assert output_line == expected_output_line


def test_line_without_equals():
    input_line = "MY_SECRET_PASSWORD"
    expected_output_line = "MY_SECRET_PASSWORD="
    output_line = strip_line(input_line)
    assert output_line == expected_output_line
